/*  CodeEdit -:-  (C)opyright 2001-2002 Andreas Engh-Halstvedt
 *  AEdit -:-     (C)opyright 2000-2002 Kristian Van Der Vliet
 *  AEdit -:-     (C)opyright 2004-2006 Jonas Jarvoll
 *  CodeEdit -:-  (C)opyright 2011-2012 David Kent
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#ifndef __BUFFER_H_
#define __BUFFER_H_

#include <util/string.h>
#include <gui/font.h>

#include <list>
#include <string>

#include <editview.h>
#include <appwindow.h>

void SetCurrentTab( int iSel );
bool AnyBufferNeedsToBeSaved();
bool MenuItemSaveAllShallBeActive();
bool SaveAllBuffers();
int CheckIfAlreadyOpened( os::String name );

class Buffer
{
public:
	Buffer( AppWindow* pcTar, char* pFileName, char* pFileBuffer );
	~Buffer();

	EditView* GetEditView() { return(pcEditView); };

	os::String &GetNameOfBuffer();
	os::String &GetRealName();
	std::string &GetTabNameOfBuffer();
	bool NeedToSave() { return(bContentChanged); };
	bool SaveStatus();
	void MakeCurrent();
	void Cut() { pcEditView->Cut(); };
	void Copy() { pcEditView->Copy(); };
	void Paste() { pcEditView->Paste(); };
	void SelectAll() { pcEditView->SelectAll(); pcEditView->Invalidate(); pcEditView->Flush(); };
	void Undo() { pcEditView->Undo(); };
	void Redo() { pcEditView->Redo(); };
	void Invoked( int32 nEvent );
	void ContentChanged( bool iContent );
	bool Save();
	bool SaveAs( const char* pFileName );
	void GotoLine( int32 iLineNo );
	void MakeFocus();
	void SetFontFamily( const char* cFam, const char* cStyle, const uint32 flags );
	void SetFontSize( float size );
	bool FindFirst( os::String zFindText, bool bCaseSensitive );
	os::String &FindNext();
	bool Replace( os::String zFText, os::String zRText );
	void ForceRedraw();
	void SetFont( os::Font* font );

	friend void SetCurrentTab( int iSel );
	friend bool AnyBufferNeedsToBeSaved();
	friend bool MenuItemSaveAllShallBeActive();
	friend bool SaveAllBuffers();
	friend int CheckIfAlreadyOpened( os::String name );

private:
	EditView* pcEditView;

	// Path to where file is stored
	os::String zFilename;

	// My new buffer number
	int iEmptyBufferNumber;

	// Set to true if the content in the buffer has changed since last save
	bool bContentChanged;

	// Font information
	os::String zFamily;				// Font family currently selected
	os::String zStyle;					// Font style currently selected
	float vSize;						// Font size currently selectes
	uint32 iFlags; 						// Font flags

	// Parameters for Find and Replace function
	os::String zFindText;				// All used for find & replace
	os::String zReplaceText;
	int32 nTextPosition, nCurrentLine, nTotalLines;
	bool bHasFind;
	bool bCaseSensitive;

protected:
	// Counter to keep track of the number of new buffers
	static int iEmptyBufferCounter;

	// Keep track of all opened buffer
	static std::list< Buffer* > pcBufferList;

	// For easy reference
	static AppWindow* pcTarget;
};

#endif /* __BUFFER_H_ */
