/*  CodeEdit -:-  (C)opyright 2001-2002 Andreas Engh-Halstvedt
 *  AEdit -:-     (C)opyright 2000-2002 Kristian Van Der Vliet
 *  AEdit -:-     (C)opyright 2004-2006 Jonas Jarvoll
 *  CodeEdit -:-  (C)opyright 2011-2012 David Kent
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <gui/image.h>
#include <gui/stringview.h>

#include <finddialog.h>
#include <buffer.h>
#include <main.h>
#include <messages.h>


FindDialog::FindDialog() : Dialog()
{
	// Create layout
	pcLayoutView = new os::LayoutView( os::Rect(), "root_layout" );
	AddChild(pcLayoutView);

	// Create basepanel	
	os::VLayoutNode* pcRoot = new os::VLayoutNode("root_node");
	pcRoot->SetVAlignment(os::ALIGN_LEFT);
	pcRoot->SetHAlignment(os::ALIGN_LEFT);
	pcLayoutView->SetRoot(pcRoot);

	// Create panels
	os::HLayoutNode* pcTopPanel = new os::HLayoutNode("");
	pcTopPanel->SetVAlignment(os::ALIGN_LEFT);
	pcTopPanel->SetHAlignment(os::ALIGN_LEFT);
	pcRoot->AddChild(pcTopPanel);

	os::HLayoutNode* pcBottomPanel = new os::HLayoutNode("");
	pcBottomPanel->SetVAlignment(os::ALIGN_LEFT);
	pcBottomPanel->SetHAlignment(os::ALIGN_LEFT);
	pcRoot->AddChild(pcBottomPanel);

	// Create close button
	os::Resources cRes( get_image_id() );
	os::BitmapImage* pcMenuBarIcon;
	pcMenuBarIcon = new os::BitmapImage();
	pcMenuBarIcon->Load( cRes.GetResourceStream("close16x16.png") );
	pcCloseButton = new os::ImageButton( os::Rect(), "close", "", new os::Message(M_BUT_FIND_CLOSE), pcMenuBarIcon );
	pcTopPanel->AddChild(pcCloseButton);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 8, 8 ) );

	// Create string
	os::StringView* pcFindString = new os::StringView( os::Rect(), "find_string", "Find" );
	pcTopPanel->AddChild(pcFindString);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create textview
	pcFindTextView = new os::TextView( os::Rect(), "", "" );
	pcFindTextView->SetMessage( new os::Message(M_BUT_FIND_TEXTVIEW) );
	pcFindTextView->SetEventMask( os::TextView::EI_ENTER_PRESSED );
	pcFindTextView->SetMinPreferredSize( 20,1 );
	pcFindTextView->SetMaxPreferredSize( 200,1 );
	pcTopPanel->AddChild(pcFindTextView);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create the find/find next button
	pcFirstButton = new os::Button( os::Rect(), "first_button", "_Find First", new os::Message(M_BUT_FIND_GO) );
	pcTopPanel->AddChild(pcFirstButton);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	pcNextButton = new os::Button( os::Rect(), "next_button", "Find _Next", new os::Message(M_BUT_FIND_NEXT) );
	pcTopPanel->AddChild(pcNextButton);

	// Create checkbox for case sensitivity
	pcCaseCheckbox = new os::CheckBox( os::Rect(),"case_checkbox", "Case Sensitive Search", new os::Message(M_VOID) );
	pcBottomPanel->AddChild(pcCaseCheckbox);

	// Set the focus
	MakeFocus();
}

void FindDialog::FrameSized( const os::Point& cDelta )
{
	os::Rect cFrame = GetBounds();
	cFrame.Resize( 2,2,-2,-2 );
	cFrame.right = cFrame.left + GetPreferredSize(false).x;
	pcLayoutView->SetFrame(cFrame);

	os::LayoutView::FrameSized(cDelta);
}

void FindDialog::AllAttached()
{
	pcFirstButton->SetTarget(this);
	pcNextButton->SetTarget(this);
	pcCloseButton->SetTarget(this);
	pcFindTextView->SetTarget(this);
	pcCaseCheckbox->SetTarget(this);

	os::LayoutView::AllAttached();
}

void FindDialog::HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_BUT_FIND_GO:
		{
			_FindFirst();
			break;
		}
		case M_BUT_FIND_NEXT:
		{
			_FindNext();
			break;
		}
		case M_BUT_FIND_CLOSE:
		{
			App::GetAppWindow()->SetDialog( NULL );
			break;
		}
		case M_BUT_FIND_TEXTVIEW:
		{
			int32 nEvent;
			
			if( pcMessage->FindInt32( "events", &nEvent ) == EOK )
			{
				if( nEvent & os::TextView::EI_ENTER_PRESSED )
					_FindFirst();			
			}
			break;
		}
		default:
			os::LayoutView::HandleMessage(pcMessage);
	}
}

// Disable/enable buttons in dialog depending on the parameter
void FindDialog::SetEnable( bool bEnable )
{
	pcFirstButton->SetEnable( bEnable );
	pcNextButton->SetEnable( bEnable );
}

void FindDialog::Init()
{
	// Set cursor on the text entry
	pcFindTextView->SelectAll();
	pcFindTextView->MakeFocus();
}

os::Point FindDialog::GetPreferredSize( bool bSize ) const
{
	os::Point p = pcLayoutView->GetPreferredSize( bSize );
	p.x += 4;
	p.y += 4;
	return p;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//
// P R I V A T E
//
///////////////////////////////////////////////////////////////////////////////////////////////////

void FindDialog::_FindFirst()
{
	Buffer* current = App::GetAppWindow()->pcCurrentBuffer;

	if( current != NULL )
	{
		os::String text = pcFindTextView->GetBuffer()[0];

		if( current->FindFirst( text, pcCaseCheckbox->GetValue() ? true : false ) )
		{
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase %s was found.", text.c_str()) );
			current->MakeFocus();
		}
		else
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase '%s' was not found.", text.c_str()) );
	}
}

void FindDialog::_FindNext()
{
	Buffer* current = App::GetAppWindow()->pcCurrentBuffer;

	if( current != NULL )
	{
		os::String text;
		text = current->FindNext();

		if( text != "" )
		{
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase %s was found.", text.c_str()) );
			current->MakeFocus();
		}
		else
		{
			App::GetAppWindow()->SetStatus( os::String("Phrase was not found.") );
		}
	}
}

void FindDialog::_Layout()
{
	os::Rect cFrame = GetBounds();
	cFrame.Resize( 2,2,-2,-2 );
	cFrame.right = cFrame.left + GetPreferredSize(false).x;

	pcLayoutView->SetFrame(cFrame);	
}

