/*  CodeEdit -:-  (C)opyright 2001-2002 Andreas Engh-Halstvedt
 *  AEdit -:-     (C)opyright 2000-2002 Kristian Van Der Vliet
 *  AEdit -:-     (C)opyright 2004-2006 Jonas Jarvoll
 *  CodeEdit -:-  (C)opyright 2011-2012 David Kent
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <gui/stringview.h>

#include <gotodialog.h>
#include <buffer.h>
#include <main.h>
#include <messages.h>


GotoDialog::GotoDialog() : Dialog()
{
	// Create layout
	pcLayoutView = new os::LayoutView( os::Rect(), "" );
	os::HLayoutNode* pcRoot = new os::HLayoutNode("");
	pcRoot->SetVAlignment(os::ALIGN_LEFT);
	pcRoot->SetHAlignment(os::ALIGN_LEFT);
	pcLayoutView->SetRoot(pcRoot);
	AddChild(pcLayoutView);

	// Create close button
	os::Resources cRes( get_image_id() );
	os::BitmapImage* pcMenuBarIcon;
	pcMenuBarIcon = new os::BitmapImage();
	pcMenuBarIcon->Load( cRes.GetResourceStream("close16x16.png") );
	pcCloseButton = new os::ImageButton( os::Rect(), "close","", new os::Message( M_BUT_GOTO_CLOSE ), pcMenuBarIcon );
	pcRoot->AddChild(pcCloseButton);

	// Create space
	pcRoot->AddChild( new os::HLayoutSpacer( "", 8, 8 ) );

	// Create string
	os::StringView* pcGotoString = new os::StringView( os::Rect(), "goto_string", "Row:" );
	pcRoot->AddChild(pcGotoString);

	// Create space
	pcRoot->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create textview
	pcLineNoTextView = new os::TextView( os::Rect(), "", "" );
	pcLineNoTextView->SetMessage( new os::Message( M_BUT_GOTO_TEXTVIEW ) );
	pcLineNoTextView->SetEventMask( os::TextView::EI_ENTER_PRESSED );
	pcLineNoTextView->SetNumeric(true);
	pcLineNoTextView->SetMinPreferredSize( 20, 1 );
	pcLineNoTextView->SetMaxPreferredSize( 200, 1 );
	pcRoot->AddChild(pcLineNoTextView);

	// Create space
	pcRoot->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create the goto button
	pcGotoButton = new os::Button( os::Rect(), "goto_button", "_Go to Line", new os::Message( M_BUT_GOTO_GOTO ) );
	pcRoot->AddChild(pcGotoButton);

	// Set the focus
	MakeFocus();
}

void GotoDialog::FrameSized( const os::Point& cDelta )
{
	_Layout();

	os::LayoutView::FrameSized(cDelta);
}

void GotoDialog::AllAttached()
{
	pcGotoButton->SetTarget(this);
	pcCloseButton->SetTarget(this);
	pcLineNoTextView->SetTarget(this);

	os::LayoutView::AllAttached();
}

void GotoDialog::HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_BUT_GOTO_GOTO:
		{
			_Goto();
			break;
		}
		case M_BUT_GOTO_CLOSE:
		{
			App::GetAppWindow()->SetDialog(NULL);
			break;
		}
		case M_BUT_GOTO_TEXTVIEW:
		{
			int32 nEvent;

			if( pcMessage->FindInt32( "events", &nEvent ) == EOK )
			{
				if( nEvent & os::TextView::EI_ENTER_PRESSED )
					_Goto();			
			}
			break;
		}
		default:
			os::LayoutView::HandleMessage(pcMessage);
	}
}

// Disable/enable buttons in dialog depending on the parameter
void GotoDialog::SetEnable( bool bEnable )
{
	pcGotoButton->SetEnable(bEnable);
}

void GotoDialog::Init()
{
	// Set cursor on the text entry
	pcLineNoTextView->SelectAll();
	pcLineNoTextView->MakeFocus();
}

os::Point GotoDialog::GetPreferredSize( bool bSize ) const
{
	os::Point p = pcLayoutView->GetPreferredSize(bSize);
	p.x += 4;
	p.y += 4;
	return p;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//
// P R I V A T E
//
///////////////////////////////////////////////////////////////////////////////////////////////////
void GotoDialog::_Goto()
{
	Buffer* current = App::GetAppWindow()->pcCurrentBuffer;

	if( current != NULL )
	{
		int line_no = atoi( pcLineNoTextView->GetBuffer()[0].c_str() );
		current->GotoLine(line_no);
		current->MakeFocus();
	}
}

void GotoDialog::_Layout()
{
	os::Rect cFrame = GetBounds();
	cFrame.Resize( 2,2,-2,-2 );
	cFrame.right = cFrame.left + GetPreferredSize(false).x;

	pcLayoutView->SetFrame(cFrame);	
}

