/*  CodeEdit -:-  (C)opyright 2001-2002 Andreas Engh-Halstvedt
 *  AEdit -:-     (C)opyright 2000-2002 Kristian Van Der Vliet
 *  AEdit -:-     (C)opyright 2004-2006 Jonas Jarvoll
 *  CodeEdit -:-  (C)opyright 2011-2012 David Kent
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details. 
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <gui/image.h>
#include <gui/stringview.h>

#include <replacedialog.h>
#include <buffer.h>
#include <main.h>
#include <messages.h>


ReplaceDialog::ReplaceDialog() : Dialog()
{
	// Create layout
	pcLayoutView = new os::LayoutView( os::Rect(), "root_layout" );
	AddChild(pcLayoutView);

	// Create basepanel	
	os::VLayoutNode* pcRoot = new os::VLayoutNode("root_node");
	pcRoot->SetVAlignment(os::ALIGN_LEFT);
	pcRoot->SetHAlignment(os::ALIGN_LEFT);
	pcLayoutView->SetRoot(pcRoot);

	// Create panels
	os::HLayoutNode* pcTopPanel = new os::HLayoutNode("");
	pcTopPanel->SetVAlignment(os::ALIGN_LEFT);
	pcTopPanel->SetHAlignment(os::ALIGN_LEFT);
	pcRoot->AddChild(pcTopPanel);

	os::HLayoutNode* pcBottomPanel = new os::HLayoutNode("");
	pcBottomPanel->SetVAlignment(os::ALIGN_LEFT);
	pcBottomPanel->SetHAlignment(os::ALIGN_LEFT);
	pcRoot->AddChild(pcBottomPanel);

	// Create close button
	os::Resources cRes(get_image_id());
	os::BitmapImage* pcMenuBarIcon;
	pcMenuBarIcon = new os::BitmapImage();
	pcMenuBarIcon->Load( cRes.GetResourceStream( "close16x16.png" ) );
	pcCloseButton = new os::ImageButton( os::Rect(), "close","", new os::Message( M_BUT_REPLACE_CLOSE ), pcMenuBarIcon );
	pcTopPanel->AddChild( pcCloseButton );

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 8, 8 ) );

	// Create string
	os::StringView* pcFindString = new os::StringView( os::Rect(), "find_string", "Find" );
	pcTopPanel->AddChild(pcFindString);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create textview
	pcFindTextView = new os::TextView( os::Rect(), "", "" );
	pcFindTextView->SetMessage( new os::Message( M_BUT_REPLACE_FIND_TEXTVIEW ) );
	pcFindTextView->SetEventMask( os::TextView::EI_ENTER_PRESSED );
	pcFindTextView->SetMinPreferredSize( 20, 1 );
	pcFindTextView->SetMaxPreferredSize( 200, 1 );
	pcTopPanel->AddChild(pcFindTextView);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create string
	os::StringView* pcReplaceString = new os::StringView( os::Rect(), "find_string", "and replace with" );
	pcTopPanel->AddChild(pcReplaceString);

	// Create space
	pcTopPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	// Create textview
	pcReplaceTextView = new os::TextView( os::Rect(), "", "" );
	pcReplaceTextView->SetMessage( new os::Message(M_BUT_REPLACE_REPLACE_TEXTVIEW) );
	pcReplaceTextView->SetEventMask( os::TextView::EI_ENTER_PRESSED );
	pcReplaceTextView->SetMinPreferredSize( 20, 1 );
	pcReplaceTextView->SetMaxPreferredSize( 200, 1 );
	pcTopPanel->AddChild(pcReplaceTextView);

	// Create checkbox for case sensitivity
	pcCaseCheckbox = new os::CheckBox( os::Rect(), "case_checkbox", "Case _Sensitive Search", new os::Message(M_VOID) );
	pcBottomPanel->AddChild(pcCaseCheckbox);

	// Create the find/find next button
	pcFirstButton = new os::Button( os::Rect(), "first_button", "_Find First", new os::Message(M_BUT_FIND_GO) );
	pcBottomPanel->AddChild(pcFirstButton);

	// Create space
	pcBottomPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	pcNextButton = new os::Button( os::Rect(), "next_button", "Find _Next", new os::Message(M_BUT_FIND_NEXT) );
	pcBottomPanel->AddChild(pcNextButton);

	// Create space
	pcBottomPanel->AddChild( new os::HLayoutSpacer( "", 4, 4 ) );

	pcReplaceButton = new os::Button( os::Rect(), "next_button", "_Replace", new os::Message(M_BUT_REPLACE_DO) );
	pcBottomPanel->AddChild(pcReplaceButton);


	// Set the focus
	MakeFocus();
}

void ReplaceDialog::FrameSized( const os::Point& cDelta )
{
	_Layout();

	os::LayoutView::FrameSized(cDelta);
}

void ReplaceDialog::AllAttached()
{
	pcFirstButton->SetTarget( this);
	pcNextButton->SetTarget(this);
	pcReplaceButton->SetTarget(this);
	pcCloseButton->SetTarget(this);
	pcFindTextView->SetTarget(this);
	pcReplaceTextView->SetTarget(this);
	pcCaseCheckbox->SetTarget(this);

	os::LayoutView::AllAttached();
}

void ReplaceDialog::HandleMessage( os::Message* pcMessage )
{
	switch( pcMessage->GetCode() )
	{
		case M_BUT_FIND_GO:
		{
			_FindFirst();
			break;
		}
		case M_BUT_FIND_NEXT:
		{
			_FindNext();
			break;
		}
		case M_BUT_REPLACE_DO:
		{
			_Replace();
			break;
		}
		case M_BUT_REPLACE_CLOSE:
		{
			App::GetAppWindow()->SetDialog(NULL);
			break;
		}
		case M_BUT_REPLACE_FIND_TEXTVIEW:
		{
			int32 nEvent;
			
			if( pcMessage->FindInt32( "events", &nEvent ) == EOK )
			{
				if( nEvent & os::TextView::EI_ENTER_PRESSED )
				{
					pcReplaceTextView->SelectAll();
					pcReplaceTextView->MakeFocus();

				}
			}
			break;
		}
		case M_BUT_REPLACE_REPLACE_TEXTVIEW:
		{
			int32 nEvent;
			
			if( pcMessage->FindInt32( "events", &nEvent ) == EOK )
			{
				if( nEvent & os::TextView::EI_ENTER_PRESSED )
					_FindFirst();			
			}
			break;
		}
		default:
			os::LayoutView::HandleMessage(pcMessage);
	}
}

// Disable/enable buttons in dialog depending on the parameter
void ReplaceDialog::SetEnable( bool bEnable )
{
	pcFirstButton->SetEnable(bEnable);
	pcNextButton->SetEnable(bEnable);
	pcReplaceButton->SetEnable(bEnable);
}

void ReplaceDialog::Init()
{
	// Set cursor on the text entry
	pcFindTextView->SelectAll();
	pcFindTextView->MakeFocus();
}

os::Point ReplaceDialog::GetPreferredSize( bool bSize ) const
{
	os::Point p = pcLayoutView->GetPreferredSize(bSize);
	p.x += 4;
	p.y += 4;
	return p;
}

///////////////////////////////////////////////////////////////////////////////////////////////////
//
// P R I V A T E
//
///////////////////////////////////////////////////////////////////////////////////////////////////

void ReplaceDialog::_FindFirst()
{
	Buffer* current = App::GetAppWindow()->pcCurrentBuffer;

	if( current != NULL )
	{
		os::String text = pcFindTextView->GetBuffer()[0];

		if( current->FindFirst( text, pcCaseCheckbox->GetValue() ? true : false ) )
		{
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase %s was found.", text.c_str()) );
			current->MakeFocus();
		}
		else
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase '%s' was not found.", text.c_str()) );
	}
}

void ReplaceDialog::_FindNext()
{
	Buffer* current = App::GetAppWindow()->pcCurrentBuffer;

	if( current != NULL )
	{
		os::String text;
		text = current->FindNext();

		if( text != "" )
		{
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase %s was found.", text.c_str()) );
			current->MakeFocus();
		}
		else
		{
			App::GetAppWindow()->SetStatus( os::String("Phrase was not found.") );
		}
	}
}

void ReplaceDialog::_Replace()
{
	Buffer* current = App::GetAppWindow()->pcCurrentBuffer;

	if( current != NULL )
	{
		os::String ftext = pcFindTextView->GetBuffer()[0];
		os::String rtext = pcReplaceTextView->GetBuffer()[0];

		if( current->Replace( ftext, rtext ) )
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase %s has been replaced with %s.", ftext.c_str(), rtext.c_str()) );
		else
			App::GetAppWindow()->SetStatus( os::String().Format("Phrase was not replaced.") );
	
		_FindNext();
	}
}				

void ReplaceDialog::_Layout()
{
	os::Rect cFrame = GetBounds();
	cFrame.Resize( 2,2,-2,-2 );
	cFrame.right = cFrame.left + GetPreferredSize( false ).x;

	pcLayoutView->SetFrame(cFrame);	
}

